#pragma once
#ifndef SERVER_COMMON_IO_POOL_EX_INCLUDED__
#define SERVER_COMMON_IO_POOL_EX_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: IOPoolEx.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2012 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/IOTools/IOPoolEx.h"
#include "JetByteTools/IOTools/IMonitorIOPool.h"

#include "JetByteTools/CoreTools/tstring.h"

///////////////////////////////////////////////////////////////////////////////
// CIOPoolEx
///////////////////////////////////////////////////////////////////////////////

class CIOPoolEx:
   public JetByteTools::IO::CIOPoolEx,
   private JetByteTools::IO::IMonitorIOPool
{
   public :

      explicit CIOPoolEx(
         ULONG numEntriesPerCall,
         ThreadCount numThreads = 0,
         bool log = true);

      CIOPoolEx(
         const CIOPoolEx &rhs) = delete;

      ~CIOPoolEx() override;

      CIOPoolEx &operator=(
         const CIOPoolEx &rhs) = delete;

   private :

      void OnIOPoolError(
         const JetByteTools::Core::_tstring &message) override;

      void OnIOPoolThreadCreated() override;
      void OnIOPoolThreadStarted() override;
      void OnIOPoolDispatch() override;
      void OnIOPoolWorkItemsDequeued(
         DWORD numWorkItems) override;
      void OnIOPoolThreadBeginProcessing() override;
      void OnIOPoolThreadEndProcessing() override;
      void OnIOPoolThreadStopped() override;
      void OnIOPoolThreadDestroyed() override;

      const bool m_log;

      const DWORD m_tlsIndex;
};

#endif // SERVER_COMMON_IO_POOL_EX_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: IOPoolEx.h
///////////////////////////////////////////////////////////////////////////////
