#pragma once
#ifndef SERVER_COMMON_SIMPLE_SERVER_SHUTDOWN_HANDLER_INCLUDED__
#define SERVER_COMMON_SIMPLE_SERVER_SHUTDOWN_HANDLER_INCLUDED__
///////////////////////////////////////////////////////////////////////////////
// File: SimpleServerShutdownHandler.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2009 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/CoreTools/tstring.h"

#include "JetByteTools/Win32Tools/SecurityDescriptorAllowAll.h"
#include "JetByteTools/Win32Tools/SecurityAttributes.h"
#include "JetByteTools/Win32Tools/ManualResetEvent.h"

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools
{
   namespace Socket
   {
      class IServerControl;
   }
}

///////////////////////////////////////////////////////////////////////////////
// CSimpleServerShutdownHandler
///////////////////////////////////////////////////////////////////////////////

class CSimpleServerShutdownHandler
{
   public :

      CSimpleServerShutdownHandler();

      explicit CSimpleServerShutdownHandler(
         JetByteTools::Socket::IServerControl &serverControl);

      CSimpleServerShutdownHandler(
         JetByteTools::Socket::IServerControl &serverControl,
         const JetByteTools::Core::_tstring &eventNamePrefix);

      CSimpleServerShutdownHandler(
         const CSimpleServerShutdownHandler &rhs) = delete;

      CSimpleServerShutdownHandler &operator=(
         const CSimpleServerShutdownHandler &rhs) = delete;

      ~CSimpleServerShutdownHandler();

      void ShutdownNow();

      void SetServerControl(
         JetByteTools::Socket::IServerControl &serverControl);

      void SetServerControl(
         JetByteTools::Socket::IServerControl &serverControl,
         const JetByteTools::Core::_tstring &eventNamePrefix);

      void WaitForShutdownRequest();

   private :

      JetByteTools::Socket::IServerControl *m_pServerControl;

      JetByteTools::Win32::CSecurityDescriptorAllowAll m_sd;

      JetByteTools::Win32::CSecurityAttributes m_sa;

      JetByteTools::Win32::CManualResetEvent *m_pServerStartedEvent;

      JetByteTools::Win32::CManualResetEvent *m_pServerStoppedEvent;

      JetByteTools::Core::_tstring m_eventNamePrefix;
};

#endif // SERVER_COMMON_SIMPLE_SERVER_SHUTDOWN_HANDLER_INCLUDED__

///////////////////////////////////////////////////////////////////////////////
// End of file: SimpleServerShutdownHandler.h
///////////////////////////////////////////////////////////////////////////////
